/*
 * Copyright (c) 2009 by Jakob Schroeter <js@camaya.net>
 * This file is part of the gloox library. http://camaya.net/gloox
 *
 * This software is distributed under a license. The full license
 * agreement can be found in the file LICENSE in this distribution.
 * This software may not be copied, modified, sold or distributed
 * other than expressed in the named license agreement.
 *
 * This software is distributed without any warranty.
 */

#ifndef CONNECTIONCOMPRESSION_H__
#define CONNECTIONCOMPRESSION_H__

#include "macros.h"

#include <gloox/gloox.h>
#include <gloox/logsink.h>
#include <gloox/connectionbase.h>
#include <gloox/connectiondatahandler.h>
#include <gloox/compressiondatahandler.h>

#include <string>

namespace gloox
{
  class CompressionDefault;

}

namespace glooxd
{

  /**
   * @brief This is an implementation of a de/compressing connection.
   *
   * There should be no need to use this class directly. It is
   * used by ClientBase internally.
   *
   * @author Jakob Schroeter <js@camaya.net>
   * @since 1.1
   */

  class GLOOXD_API ConnectionCompression : public gloox::CompressionDataHandler,
                                           public gloox::ConnectionBase,
                                           gloox::ConnectionDataHandler
  {
    public:
      /**
       * Constructs a new ConnectionCompression object.
       * @param cdh The ConnectionDataHandler that will be notified of events from this connection
       * @param conn A transport connection. It should be configured to connect to
       * the server and port you wish to make the encrypted connection to.
       * ConnectionCompression will own the transport connection and delete it in its destructor.
       * @param logInstance The log target. Obtain it from ClientBase::logInstance().
       */
      ConnectionCompression( gloox::ConnectionDataHandler* cdh, gloox::ConnectionBase* conn,
                             const gloox::LogSink& log );

      /**
       * Constructs a new ConnectionCompression object.
       * @param conn A transport connection. It should be configured to connect to
       * the server and port you wish to make the encrypted connection to.
       * ConnectionCompression will own the transport connection and delete it in its destructor.
       * @param logInstance The log target. Obtain it from ClientBase::logInstance().
       */
      ConnectionCompression( gloox::ConnectionBase* conn, const gloox::LogSink& log );

      /**
       * Virtual Destructor.
       */
      virtual ~ConnectionCompression();

      /**
       * Sets the transport connection.
       * @param connection The transport connection to use.
       */
      void setConnectionImpl( gloox::ConnectionBase* connection );

      // reimplemented from ConnectionBase
      virtual gloox::ConnectionError connect();

      // reimplemented from ConnectionBase
      virtual gloox::ConnectionError recv( int timeout = -1 );

      // reimplemented from ConnectionBase
      virtual bool send( const std::string& data );

      // reimplemented from ConnectionBase
      virtual gloox::ConnectionError receive();

      // reimplemented from ConnectionBase
      virtual void disconnect();

      // reimplemented from ConnectionBase
      virtual void cleanup();

      // reimplemented from ConnectionBase
      virtual void getStatistics( int& totalIn, int& totalOut );

      // reimplemented from ConnectionDataHandler
      virtual void handleReceivedData( const gloox::ConnectionBase* connection, const std::string& data );

      // reimplemented from ConnectionDataHandler
      virtual void handleConnect( const gloox::ConnectionBase* connection );

      // reimplemented from ConnectionDataHandler
      virtual void handleDisconnect( const gloox::ConnectionBase* connection, gloox::ConnectionError reason );

      // reimplemented from ConnectionDataHandler
      virtual gloox::ConnectionBase* newInstance() const;

      // reimplemented from CompressionDataHandler
      virtual void handleCompressedData( const std::string& data );

      // reimplemented from CompressionDataHandler
      virtual void handleDecompressedData( const std::string& data );


    private:
      gloox::ConnectionBase* m_connection;
      gloox::CompressionDefault* m_compression;
      const gloox::LogSink & m_log;

  };

}

#endif // CONNECTIONCOMPRESSION_H__
